io.stdout:setvbuf("no")

local BASE = (...):match("(.-)[^%.]+$")

local FONTSIZE = 16
local CANVAS_X, CANVAS_Y = 64, 64
local _XTOOLS, _YTOOLS = 4, 4

FONTS = {
	 
  small = love.graphics.newFont(12),
	normal = love.graphics.newFont(FONTSIZE),
  big = love.graphics.newFont(32),
}


local _TOOLS = {
	{ "", "Line", "Curved Line", "" },
	{ "Ellipse", "", "", "Rectangle" },
	{ "Filled Ellipse", "", "", "Filled Rectangle" },
	{ "Eraser", "Brush", "Fill", "" }
}
local _selectedTool = _TOOLS[4][2]

local _lg = love.graphics

local _width = CANVAS_X -- _lg.getWidth()
local _height = CANVAS_Y -- _lg.getHeight()

local _gridSize = 1
local _pixelSize = 1

local _currData = null

local _paintImage = null

local _paintImageData = null 
local _gridCanvas = null

local _toolCanvas = love.graphics.newImage( BASE .. "imgs/toolbar.png" )
local _showToolBar = false
local	_toolBarX, _toolBarY = 0,0
		

local _tmpImageData = null
local _tmpImage = null
local _tmpDraw = false



local _gridColor = { 255,255,255, 128 }
local _color = { 0, 0, 255, 255 }
local _whiteColor = {255,255,255,255}
local _eraseColor = { 0,0,0,0 }

_lg.setNewFont(FONTSIZE)

local _paintObjects = {}

local _undos = {}
local _undoIndex = 0
local _undoCount = 0

local _panning = false
local _lastPanX, _lastPanY = 0, 0

local _lastMouseX, _lastMouseY = 0,0
local _leftDown = false
local _ctrlDown = false

local _scaler = 8
local _zoomValue = _scaler - 1
local _scalePointX = math.floor( ( _lg.getWidth()  - CANVAS_X * _scaler ) / 2 )
local _scalePointY = math.floor( ( _lg.getHeight() - CANVAS_Y * _scaler ) / 2 )

local _toolString = (_selectedTool .. ":" .. _pixelSize)

local _fileName = ""

local _gui = null

local function clearImage( x, y, r,g,b,a )
	return 0,0,0,0
end

local function copyData(x, y, r,g,b,a )
	color = { _currData:getPixel( x, y ) } 
	if( color[4] == 0 ) then return r,g,b,a end
	return unpack( color )

end

local function createGrid()
	if( _gridSize <= 0 ) then return end

	_lg.setCanvas( _gridCanvas )
	_gridCanvas:clear()
	_lg.setLineStyle('rough')
	_lg.setLineWidth(1)
	
	local startX = math.floor(_scalePointX) - 1 * CANVAS_X * _scaler 
	local startY = math.floor(_scalePointY) - 1 * CANVAS_Y * _scaler
	
	local endX, endY = math.floor( startX + CANVAS_X * _scaler * 3), 
		math.floor( startY + CANVAS_Y * _scaler * 3 )
	local tileSize = math.floor( _gridSize * _scaler )
	
	local i, j = math.floor( startX ), math.floor( startY )
	

	_lg.setColor( _gridColor )
	
	for i = startX, endX, CANVAS_X * _scaler / 4 do
		_lg.line(  i + _pixelSize / 2, startY, i + _pixelSize / 2, endY )			
	end
	
	for j = startY, endY, CANVAS_Y * _scaler / 4 do
		_lg.line( startX, j + _pixelSize / 2, endX, j + _pixelSize / 2 )
	end
	
	_lg.setColor( _whiteColor )
		
	for i = startX, endX, CANVAS_X * _scaler do
		_lg.line(  i + _pixelSize / 2, startY, i + _pixelSize / 2, endY )			
	end
	
	for j = startY, endY, CANVAS_Y * _scaler do
		_lg.line( startX, j + _pixelSize / 2, endX, j + _pixelSize / 2 )
	end

	_lg.setColor( _gridColor )

	if( startX < 0 ) 			then 
		startX = 0
		i = _scalePointX % tileSize
	elseif( startY < 0 ) 	then 
		startY = 0 
		j = math.floor( _scalePointY % tileSize )
	end
	
	if( endX > _lg.getWidth()  ) 			then endX = _lg.getWidth()
	elseif( endY > _lg.getHeight() ) 	then endY = _lg.getHeight() end 
	
	for i = i, endX, tileSize do
		_lg.line(  i + _pixelSize / 2, startY, i + _pixelSize / 2, endY )			
	end
	for j = j, endY, tileSize do
		_lg.line( startX, j + _pixelSize / 2, endX, j + _pixelSize / 2 )
	end

	
	_lg.setLineStyle('rough')
	_lg.setCanvas()
	_lg.setLineWidth(1)
end

local function drawPoint( x, y, color, size )
	local tmp = math.floor( size / 2 )
	for j = 0, size - 1 do		
		for i = 0, size - 1 do
			_currData:setPixel( ( math.floor( x ) - tmp + i ) % CANVAS_X,
															( math.floor( y ) - tmp + j ) % CANVAS_Y,
															unpack(color) )
			--_lg.point( ( math.floor( x ) + i ) % CANVAS_X + 0.5, ( math.floor( y ) + j ) % CANVAS_Y + 0.5 )
		end
	end
end




local function drawRectangle(x1,y1 , x2,y2, filled)
	
		local xDir = 1
		local yDir = 1
		
		if( x2 < x1 ) then xDir = -1 end
		if( y2 < y1 ) then yDir = -1 end
		
		if( filled ) then
			for y = y1, y2, yDir do
				for x = x1, x2, xDir do
					drawPoint(x, y, _color, 1)
				end
			end
			
			
		else		
			for y = y1, y2, yDir do
				drawPoint(x1, y,_color, _pixelSize)
				drawPoint(x2, y,_color, _pixelSize)
			end
			for x = x1, x2, xDir do
				drawPoint(x, y1, _color, _pixelSize)
				drawPoint(x, y2, _color, _pixelSize)	
			end
		end
end


local function drawLine( x1,y1 , x2,y2, color )
	-- since math.floor rounds down always -> -0.2 => -1
	local dx = math.floor( x2 - x1 )
	local dy = math.floor( y2 - y1 )
	
	if( dx < 0 ) then dx = dx + 1 end
	if( dy < 0 ) then dy = dy + 1 end
	
	local yDir = math.floor(dy / math.abs( dy ))
	local xDir = math.floor(dx / math.abs( dx ))
	
	dx = math.abs( dx )
	dy = math.abs( dy )
	
	local x = math.floor( x1 )
	local y = math.floor( y1 )
	
	_lg.point( x % CANVAS_X, y % CANVAS_Y )
	if( dx == 0 ) then
		for y = y, math.floor( y2 ), yDir do
			drawPoint( x, y, color,_pixelSize )
		end
	elseif( dy == 0 ) then
		for x = x, math.floor( x2 ), xDir do
			drawPoint( x, y, color,_pixelSize )
		end
	else
		if( dx >= dy ) then
			local indexer = dx / 2
			for x = x, math.floor( x2 ), xDir do
				drawPoint( x, y, color, _pixelSize )
				indexer = indexer + dy
				if( indexer >= dx ) then
					indexer = indexer - dx
					y = y + yDir
				end
			end
			else
				local indexer = dy / 2
				for y = y, math.floor( y2 ), yDir do
					drawPoint( x, y, color, _pixelSize )
					indexer = indexer + dx
					if( indexer >= dy ) then
						indexer = indexer - dy
						x = x + xDir
					end
				end
			
			end
		
	end
end


local function fillArea( x1,y1, color, floodarray )
	x1, y1 = x1 % CANVAS_X, y1 % CANVAS_Y
	
	floodarray[y1] = floodarray[y1] or {}
	if( floodarray[y1][x1] ~= null ) then return end;
	floodarray[y1][x1] = true

	
	
	local c = { _paintImageData:getPixel(x1,y1) }
	if( c == null or color == null ) then return end
	for i = 1, #color do
		if( c[i] ~= color[i] ) then return end

	end
	
	drawPoint( x1,y1, _color, 1 )
	
	floodarray[y1+1] = floodarray[y1+1] or {}
	floodarray[y1-1] = floodarray[y1-1] or {}
	
	if( floodarray[y1][x1+1] == null )then fillArea( x1+1,y1, color, floodarray ) end
	if( floodarray[y1][x1-1] == null )then fillArea( x1-1,y1, color, floodarray ) end
	if( floodarray[y1+1][x1] == null )then fillArea( x1,y1+1, color, floodarray ) end
	if( floodarray[y1-1][x1] == null )then fillArea( x1,y1-1, color, floodarray ) end
end


local function glitchFillArea( x1,y1, color, floodarray )
	x1, y1 = x1 % CANVAS_X, y1 % CANVAS_Y
	
	floodarray[y1] = floodarray[y1] or {}
	if( floodarray[y1][x1] ~= null ) then return end;
	floodarray[y1][x1] = true

	
	if( x1 <0 or y1<0 or x1 >= _paintImageData:getWidth() or y1 >= _paintImageData:getHeight() ) then
		return
	end
	
	local c = { _paintImageData:getPixel(x1,y1) }
	if( c == null or color == null ) then return end
	for i = 1, #color do
		if( c[i] ~= color[i] ) then return end
	end
	
	drawPoint( x1,y1, _color, 1 )
	
	floodarray[y1+1] = floodarray[y1+1] or {}
	floodarray[y1-1] = floodarray[y1-1] or {}
	
	if( floodarray[y1][x1+1] == null )then fillArea( x1+1,y1, color, floodarray ) end
	if( floodarray[y1][x1-1] == null )then fillArea( x1-1,y1, color, floodarray ) end
	if( floodarray[y1+1][x1] == null )then fillArea( x1,y1+1, color, floodarray ) end
	if( floodarray[y1-1][x1] == null )then fillArea( x1,y1-1, color, floodarray ) end
end


local function	drawEllipse(x1,y1, x2,y2, filled )	
	local oldPixelSize = _pixelSize
	if( filled ) then _pixelSize = 1 end
	
	if( x1 == x2 ) then 
		drawLine(x1,y2,x2,( 2* y1 - y2 ),_color ) 
		_pixelSize = oldPixelSize
	elseif( y1 == y2 ) then 
		drawLine(x1,y1,( 2*x1 - x2 ),y2,_color )  
		_pixelSize = oldPixelSize
	end
	
	--x2 = x2 + 0.5
	--y2 = y2 + 0.5
	
	
	local a = math.abs(x2-x1) + 0.5
	local b = math.abs(y2-y1) + 0.5
	local a2 = a*a
	local b2 = b*b
	local y = 0
	local tmp = 0
	
	
	
	for x = - a + 0.5, 0 do
		tmp = x*x/a2
		while( tmp + (y+1)*(y+1) / b2 < 1 ) do 
			if( filled ) then
				local oldPixelSize = _pixelSize
				
				drawLine( x1-x,y1-y, x1+x,y1-y, _color )
				drawLine( x1-x,y1+y, x1+x,y1+y, _color )
				
			else
				
				drawPoint( x1 + x, y1 + y, _color, _pixelSize )
				drawPoint( x1 + x, y1 - y, _color, _pixelSize )
				drawPoint( x1 - x, y1 + y, _color, _pixelSize )
				drawPoint( x1 - x, y1 - y, _color, _pixelSize )
			end
			y = y + 1 
		end
		if( filled ) then
			drawLine( x1-x,y1-y, x1+x,y1-y, _color )
			drawLine( x1-x,y1+y, x1+x,y1+y, _color )
				
		else
				
			drawPoint( x1 + x, y1 + y, _color, _pixelSize )
			drawPoint( x1 + x, y1 - y, _color, _pixelSize )
			drawPoint( x1 - x, y1 + y, _color, _pixelSize )
			drawPoint( x1 - x, y1 - y, _color, _pixelSize )
		end
		
	end
	
	
		_pixelSize = oldPixelSize
	
end

local function drawOnCanvas(x, y, color)
	--_lg.setCanvas(_paintImageData)

	--_lg.setColor( _color )
	
	
	--_lg.rectangle('fill', x, y, 10, 10)	
	if( x == _lastMouseX and y == _lastMouseY ) then 
		drawPoint( x, y, color, _pixelSize )
	else
		
		drawLine( x,y ,_lastMouseX, _lastMouseY, color )
		
	
		
	end
  
	_lg.setCanvas()
	_undoCount = 0
	
end



local function drawFromCanvasToCanvas( source, dest, doNotClear )
	if( not doNotClear ) then 
		dest:clear()
	end
	dest:renderTo( function()	
		dest:setFilter("nearest", "nearest")
	  _lg.setColor(255,255,255)
    _lg.draw(source, 0, 0)
		end
	)

end

local function createUndo()
	_undoIndex = _undoIndex + 1
	local newImageData = _undos[ _undoIndex ]
	
	
	if( not newImageData ) then
		newImageData = love.image.newImageData( _width, _height ) 
		_undos[_undoIndex] = newImageData
	end
	
	newImageData:paste( _paintImageData, 0,0,0,0, _width, _height )
	
	--drawFromCanvasToCanvas(  _paintImageData, newCanvas )

end



local function doUndo()
	if( _undoIndex < 1 ) then return end
	

	_undoIndex = _undoIndex - 1
	_undoCount = _undoCount + 1
	
	if( _undoIndex == 0 ) then 
		_paintImageData:mapPixel( clearImage )
		-- just to make the canvas actually clear, flushes the gpu
		--_paintImageData:getPixel(0,0)
		return
	end

	_paintImageData:paste( _undos[ _undoIndex ], 0,0,0,0, _width, _height )
	--drawFromCanvasToCanvas( _undos[ _undoIndex ], _paintImageData )
end

local function doRedo()
	if( _undoIndex >= #_undos or _undoCount <= 0 ) then return end
	_undoCount = _undoCount - 1
	_undoIndex = _undoIndex + 1
	_paintImageData:paste( _undos[ _undoIndex ], 0,0,0,0, _width, _height )
	--drawFromCanvasToCanvas( _undos[ _undoIndex ], _paintImageData )

end


local function 	zoom( zoomValue )
	_zoomValue = _zoomValue + zoomValue
	local lastScale = _scaler
	
	if( _zoomValue < 0 ) then 
		_zoomValue = 0
		
		return
		
	else 
		_scaler =  _zoomValue + 1 
	end
	
	local x, y, scaleValue = love.mouse.getX(), love.mouse.getY(), _scaler / lastScale
	
	
	_scalePointX = math.floor( x - ( x - _scalePointX ) * scaleValue )
	_scalePointY = math.floor( y - ( y - _scalePointY ) * scaleValue )
	createGrid()
end

local function getScaledPosition()
	return math.floor(( love.mouse.getX() - _scalePointX ) / _scaler),
				 math.floor(( love.mouse.getY() - _scalePointY ) / _scaler)
end


function love.textinput(text)
             
    -- your code
 
    _gui.textinput( text )
 
end

function love.load()
	
	
	
	--if arg[#arg] == "-debug" then require("mobdebug").start() end 
	_lg.setLineStyle('rough')
	_lg.setPointStyle('rough')
	
	
	_paintImageData = love.image.newImageData( _width, _height )
	_paintImage = _lg.newImage( _paintImageData )
	_paintImage:setFilter("nearest","nearest")
	--_paintImageData = _lg.newCanvas( _width, _height ) 
	--_paintImageData:setFilter("nearest", "nearest")
	--_paintImageData:clear()
	
	_tmpImageData = love.image.newImageData( _width, _height ) --= _lg.newCanvas( _width, _height ) 
	_tmpImage = _lg.newImage( _tmpImageData )
	--_tmpImageData:setFilter("nearest", "nearest")
	--_tmpImageData:clear()
	
	
	
	
	_gridCanvas = _lg.newCanvas( _lg.getWidth(), _lg.getHeight() ) 
	_gridCanvas:clear()
	_gridCanvas:setFilter("nearest", "nearest")
	createGrid()
	
	
	_toolCanvas:setFilter("nearest", "nearest")
	
	
	
	_gui = require( BASE .. "gui" )
	_gui.init()
	
end

function love.update(dt)
	if love.mouse.isDown("r") then
		_color = { math.random(255), math.random(255), math.random(255) }
	end
	--[[
	if _leftDown and love.keyboard.isDown("lshift","rshift") then
		
		_lg.setColor(_color)
		local x, y = getScaledPosition()
		_tmpImageData:mapPixel( clearImage )
		_tmpDraw = true
		
		_lg.setCanvas()
		
		
		
	elseif _leftDown and love.keyboard.isDown("lctrl", "rctrl") then
		local x, y = getScaledPosition()
		--_lg.rectangle( 'fill', x1,y1, x2-x1,y2-y1 )
		
		--drawLine( _lastMouseX,_lastMouseY, x,y  )
		]]
	if _leftDown then
		local x, y = getScaledPosition()
		x,y = math.floor(x), math.floor(y)
		_lg.setColor(_color)		
		_tmpDraw = true
		
		_currData = _tmpImageData
		
		
		if( _selectedTool == "Brush" ) then
			_currData = _paintImageData
			_tmpDraw = false
			drawOnCanvas( x, y, _color )
			_lastMouseX, _lastMouseY = x, y
		elseif( _selectedTool == "Eraser" ) then
			_currData = _paintImageData
			drawOnCanvas( x, y, _eraseColor )
			_tmpDraw = false
			_lastMouseX, _lastMouseY = x, y
		elseif( _selectedTool == "Fill" )then
			_currData = _paintImageData
			_lg.setColor( _color )
			fillArea( x%CANVAS_X, y%CANVAS_Y, { _paintImageData:getPixel(x%CANVAS_X,y%CANVAS_Y) }, {}  )
					
		elseif( _selectedTool == "Filled Ellipse" ) then			
			_tmpImageData:mapPixel(clearImage)
			if( love.keyboard.isDown("lctrl", "rctrl") ) then
				drawEllipse( _lastMouseX, _lastMouseY, x, y, true )
			else
				drawEllipse( (x + _lastMouseX ) / 2, ( y + _lastMouseY ) / 2, x, y, true )
			end
		elseif( _selectedTool == "Ellipse" ) then
			_tmpImageData:mapPixel(clearImage)
			if( love.keyboard.isDown("lctrl", "rctrl") ) then
				drawEllipse( _lastMouseX, _lastMouseY, x, y, false )
			else
				drawEllipse( (x + _lastMouseX ) / 2, ( y + _lastMouseY ) / 2, x, y, false )
			end
		elseif( _selectedTool == "Filled Rectangle" ) then
			_tmpImageData:mapPixel(clearImage)
			drawRectangle(_lastMouseX,_lastMouseY, x, y, true)
		
		elseif( _selectedTool == "Rectangle" ) then
			_tmpImageData:mapPixel(clearImage)
			drawRectangle(_lastMouseX,_lastMouseY, x, y, false)
		
		elseif( _selectedTool == "Line" ) then
			_tmpImageData:mapPixel(clearImage)
			drawLine( _lastMouseX,_lastMouseY, x,y, _color  )

		end
	--	_lg.setCanvas()
	
				
	elseif _panning then
		local x, y = love.mouse.getPosition()
		_scalePointX, _scalePointY = _scalePointX + ( x - _lastPanX ), _scalePointY + ( y - _lastPanY )
		_lastPanX, _lastPanY = x, y
		createGrid()
	end
	
	_gui.update(dt)
end


function love.draw()
	_lg.setFont( FONTS["normal"] )
	_lg.setColor( _whiteColor )
	_paintImage = _lg.newImage( _paintImageData )
	_paintImage:setFilter("nearest", "nearest")
	
	if( _tmpDraw ) then 
		_tmpImage = _lg.newImage( _tmpImageData )
		_tmpImage:setFilter("nearest", "nearest")
		end
	local xSize, ySize = _scaler * CANVAS_X, _scaler * CANVAS_Y
	
	for j = -1, 1 do
		for i = -1, 1 do
			_lg.draw( _paintImage, _scalePointX + i * xSize, _scalePointY + j * ySize ,0, _scaler, _scaler )
			if( _tmpDraw ) then
				_lg.draw( _tmpImage, _scalePointX + i * xSize, _scalePointY + j * ySize ,0, _scaler, _scaler )
			end
		end
	
	end
	_lg.draw(_gridCanvas, 0,0 ) --i * xSize, j * ySize)
	
	_lg.print( _toolString )
	
	
	
	if( _showToolBar ) then
		_lg.draw( _toolCanvas, _toolBarX, _toolBarY )
	end
	
	
	_lg.print( love.timer.getFPS(), 0, _lg.getHeight() - FONTSIZE )
	
	
	
	_gui.draw()
end

function love.resize(w, h)
	
	_gridCanvas = _lg.newCanvas( _lg.getWidth(), _lg.getHeight() ) 
	_gridCanvas:setFilter("nearest", "nearest")
	_gridCanvas:clear()
	createGrid()
	
end


function love.mousepressed(x, y, button)

	if button == 'l' then
		_tmpImageData:mapPixel( clearImage )
		_leftDown = true
		_lastMouseX, _lastMouseY = getScaledPosition()
	elseif button == "m" then
		_lastPanX, _lastPanY = love.mouse.getPosition()
		_panning = true
	elseif button == "r" then
		_showToolBar = true
		_toolBarX, _toolBarY = x - _toolCanvas:getWidth() / 2, y - _toolCanvas:getHeight() / 2
		if( _toolBarX >  _lg.getWidth() - _toolCanvas:getWidth() / 2 ) then
			_toolBarX = _lg.getWidth() - _toolCanvas:getWidth() / 2
		end
		if( _toolBarY > _lg.getHeight() - _toolCanvas:getHeight() / 2 ) then
			_toolBarY = _lg.getHeight() - _toolCanvas:getHeight() /2
		end
		
	elseif button == "wu" then
		zoom(1)
	elseif button == "wd" then
		zoom(-1)
	end
	
	
	_gui.mousepressed(x, y, button)
end

function love.mousereleased( x, y, button )
	if button == 'l' then
		_leftDown = false
		
		if( _tmpDraw ) then
			_currData = _tmpImageData
			_paintImageData:mapPixel( copyData )
			--_paintImageData:paste( _tmpImageData, 0,0,0,0, _width, _height )
			--drawFromCanvasToCanvas(_tmpImageData, _paintImageData, true)
		end
		
		_tmpDraw = false
		createUndo()

	elseif button == "m" then
		_panning = false
	elseif button == "r" then
		_showToolBar = false
		
		local toolX = math.floor(( x - _toolBarX ) / ( _toolCanvas:getWidth()  / _XTOOLS )) + 1
		local toolY = math.floor(( y - _toolBarY ) / ( _toolCanvas:getHeight() / _YTOOLS )) + 1
		
		if( toolX <= 0 or toolX > _XTOOLS or toolY <= 0 or toolY > _YTOOLS ) then return end
		
		if( _TOOLS[toolY][toolX] == "" ) then return end
		
		_selectedTool = _TOOLS[toolY][toolX]
		_toolString = (_selectedTool .. ":" .. _pixelSize)
	end
	_gui.mousereleased(x, y, button)
end

function love.touchreleased( id, x, y, p )
	if( id == 0 ) then

		_gui.mousereleased( math.floor( x * love.window.getWidth() ) , 
				math.floor( love.window.getHeight() * y ),"l")
	end
	
end


function love.keypressed(key, unicode)
	
	if( _gui.keypressed(key, unicode) ) then return end

	
	if key == "z" and love.keyboard.isDown("lctrl","rctrl") then
		doUndo()
	elseif key == "y" and love.keyboard.isDown("lctrl","rctrl") then
		doRedo()

	elseif key == "z" then
		local x, y = getScaledPosition()
		print( "x1: " .. _lastMouseX .. " ,y1: " .. _lastMouseY .. " - x2: " .. x .. ", y2: " .. y )
		print( "diff: " .. _lastMouseX - x .. "," .. _lastMouseY - y )
	elseif key == "t" then
		_pixelSize = _pixelSize + 1
		_toolString = (_selectedTool .. ":" .. _pixelSize)
	elseif key == "y" then
		_pixelSize = _pixelSize - 1
		if( _pixelSize <= 0 ) then _pixelSize = 1 end
		_toolString = (_selectedTool .. ":" .. _pixelSize)
		
		
		
	elseif key == "1" then
		zoom( 1 )
	elseif key == "2" then		
		zoom( -1 )
		
	elseif key == "q" then
		_gridSize = _gridSize + 1
		createGrid()
	elseif key == "e" then
		_gridSize = _gridSize - 1
		if( _gridSize < 1) then _gridSize = 1 end
		createGrid()
		
	elseif key == "a" then
		_paintImageData:mapPixel(clearImage)
		--_paintImageData:clear(0,0,0)
		-- just to make the canvas actually clear, flushes the gpu
		--_paintImageData:getPixel(0,0)
	elseif key == "i" then
		_gui.showInput()
	
	
	end
	
	 
end


function love.keyreleased(key)
 
    -- your code
 
  _gui.keyreleased(key)
 
end

function loadFile( fileName )
	fileName = string.lower( fileName )
	if( fileName:sub( -4 ) == ".png" ) then
		if love.filesystem.exists( fileName ) then 
			print("exists") 
		end
	else
		if love.filesystem.exists( fileName .. ".png" ) then 
			print("exists") 
			local image = _lg.newImage( fileName .. ".png" )
			_paintImageData = image:getData()

			createUndo()
		end
	end
end

function saveFile( fileName )
	fileName = string.lower( fileName )
	if love.filesystem.exists( fileName ) then print("exists") end
	if( fileName:sub( -4 ) == ".png" ) then
		_paintImageData:encode( fileName )
	else
		_paintImageData:encode( fileName .. ".png" )
	end
end
